@@ -3,7 +3,7 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <ctype.h>
 #include <errno.h>
 
+#include "c-ctype.h"
 #include "gettext.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
@@ -78,8 +79,11 @@ account_t *account_new(const char *conffile, const char *id)
     a->tls_key_file = NULL;
     a->tls_cert_file = NULL;
     a->tls_trust_file = NULL;
+    a->tls_crl_file = NULL;
     a->tls_nocertcheck = 0;
     a->tls_force_sslv3 = 0;
+    a->tls_min_dh_prime_bits = -1;
+    a->tls_priorities = NULL;
     return a;
 }
 
@@ -124,8 +128,13 @@ account_t *account_copy(account_t *acc)
 	    acc->tls_cert_file ? xstrdup(acc->tls_cert_file) : NULL;
 	a->tls_trust_file = 
 	    acc->tls_trust_file ? xstrdup(acc->tls_trust_file) : NULL;
+	a->tls_crl_file = 
+	    acc->tls_crl_file ? xstrdup(acc->tls_crl_file) : NULL;
 	a->tls_nocertcheck = acc->tls_nocertcheck;
 	a->tls_force_sslv3 = acc->tls_force_sslv3;
+	a->tls_min_dh_prime_bits = acc->tls_min_dh_prime_bits;
+	a->tls_priorities =
+	    acc->tls_priorities ? xstrdup(acc->tls_priorities) : NULL;
     }
     return a;
 }
@@ -155,6 +164,8 @@ void account_free(void *a)
 	free(p->tls_key_file);
 	free(p->tls_cert_file);
 	free(p->tls_trust_file);
+	free(p->tls_crl_file);
+	free(p->tls_priorities);
 	free(p);
     }
 }
@@ -292,7 +303,7 @@ int check_auth_arg(char *arg)
     	l = strlen(arg);
 	for (i = 0; i < l; i++)
 	{
-	    arg[i] = toupper((unsigned char)arg[i]);
+	    arg[i] = c_toupper((unsigned char)arg[i]);
 	}
 	return 0;
     }
@@ -406,6 +417,12 @@ void override_account(account_t *acc1, account_t *acc2)
 	acc1->tls_trust_file = 
 	    acc2->tls_trust_file ? xstrdup(acc2->tls_trust_file) : NULL;
     }
+    if (acc2->mask & ACC_TLS_CRL_FILE)
+    {
+	free(acc1->tls_crl_file);
+	acc1->tls_crl_file = 
+	    acc2->tls_crl_file ? xstrdup(acc2->tls_crl_file) : NULL;
+    }
     if (acc2->mask & ACC_TLS_NOCERTCHECK)
     {
 	acc1->tls_nocertcheck = acc2->tls_nocertcheck;
@@ -414,6 +431,16 @@ void override_account(account_t *acc1, account_t *acc2)
     {
 	acc1->tls_force_sslv3 = acc2->tls_force_sslv3;
     }
+    if (acc2->mask & ACC_TLS_MIN_DH_PRIME_BITS)
+    {
+	acc1->tls_min_dh_prime_bits = acc2->tls_min_dh_prime_bits;
+    }
+    if (acc2->mask & ACC_TLS_PRIORITIES)
+    {
+	free(acc1->tls_priorities);
+	acc1->tls_priorities = acc2->tls_priorities 
+	    ? xstrdup(acc2->tls_priorities) : NULL;
+    }
     acc1->mask |= acc2->mask;
 }
 
@@ -454,6 +481,11 @@ int check_account(account_t *acc, int retrmail, char **errstr)
 		    "a disabled tls_certcheck"));
 	return CONF_ESYNTAX;
     }
+    if (acc->tls_crl_file && !acc->tls_trust_file)
+    {
+	*errstr = xasprintf(_("tls_crl_file requires tls_trust_file"));
+      	return CONF_ESYNTAX;
+    }
     if (retrmail && acc->delivery_method == -1)
     {
 	*errstr = xasprintf(_("no delivery information"));
@@ -1035,6 +1067,12 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 	    free(acc->tls_trust_file);
 	    acc->tls_trust_file = (*arg == '\0') ? NULL : expand_tilde(arg);
 	}
+	else if (strcmp(cmd, "tls_crl_file") == 0)
+	{
+	    acc->mask |= ACC_TLS_CRL_FILE;
+	    free(acc->tls_crl_file);
+	    acc->tls_crl_file = (*arg == '\0') ? NULL : expand_tilde(arg);
+	}
 	else if (strcmp(cmd, "tls_certcheck") == 0)
 	{
 	    acc->mask |= ACC_TLS_NOCERTCHECK;
@@ -1075,6 +1113,39 @@ int read_conffile(const char *conffile, FILE *f, list_t **acc_list,
 		break;
 	    }
 	}
+	else if (strcmp(cmd, "tls_min_dh_prime_bits") == 0)
+	{
+	    acc->mask |= ACC_TLS_MIN_DH_PRIME_BITS;
+	    if (*arg == '\0')
+	    {
+		acc->tls_min_dh_prime_bits = -1;
+	    }
+	    else
+	    {
+		acc->tls_min_dh_prime_bits = get_non_neg_int(arg);
+		if (acc->tls_min_dh_prime_bits < 1)
+		{
+		    *errstr = xasprintf(
+			    _("line %d: invalid argument %s for command %s"),
+			    line, arg, cmd);
+		    e = CONF_ESYNTAX;
+		    break;
+		}
+	    }
+	}
+	else if (strcmp(cmd, "tls_priorities") == 0)
+	{
+	    acc->mask |= ACC_TLS_PRIORITIES;
+	    free(acc->tls_priorities);
+	    if (*arg == '\0')
+	    {
+		acc->tls_priorities = NULL;
+	    }
+	    else
+	    {
+		acc->tls_priorities = xstrdup(arg);
+	    }
+	}
 	else if (strcmp(cmd, "only_new") == 0)
 	{
 	    acc->mask |= ACC_ONLY_NEW;
