@@ -3,8 +3,11 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009
  * Martin Lambers <marlam@marlam.de>
+ * Dimitrios Apostolou <jimis@gmx.net> (UID handling)
+ * Jay Soffian <jaysoffian@gmail.com> (Mac OS X keychain support)
+ * Satoru SATOH <satoru.satoh@gmail.com> (GNOME keyring support)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,27 +42,18 @@ extern int optind;
 #include <sys/stat.h>
 #include <unistd.h>
 #include <sysexits.h>
+#include <signal.h>
+#include <netdb.h>
+#include <arpa/inet.h>
+#include <fcntl.h>
 #ifdef ENABLE_NLS
 # include <locale.h>
 #endif
-#if HAVE_SIGACTION
-# include <signal.h>
+#ifdef HAVE_GNOMEKEYRING
+# include <gnome-keyring.h>
 #endif
-#ifdef W32_NATIVE
-# include <io.h>
-# include <direct.h>
-# include <fcntl.h>
-# include <windows.h>
-# include <winsock2.h>
-# define mkdir(dir, mode) _mkdir(dir)
-#elif defined DJGPP
-# include <io.h>
-# include <fcntl.h>
-# include <netdb.h>
-# include <arpa/inet.h>
-#else
-# include <netdb.h>
-# include <arpa/inet.h>
+#ifdef HAVE_MACOSXKEYRING
+# include <Security/Security.h>
 #endif
 
 #include "c-ctype.h"
@@ -84,16 +78,19 @@ extern int optind;
 /* Default file names. */
 #ifdef W32_NATIVE
 #define CONFFILE	"mpoprc.txt"
-#define UIDLSFILE	"mpop_uidls/%U_at_%H.txt"
-#define NETRCFILE	"netrc.txt"
+#define UIDLSFILE	"mpop_uidls\\%U_at_%H.txt"
+#define USERNETRCFILE	"netrc.txt"
+#define SYSNETRCFILE	"netrc.txt"
 #elif defined (DJGPP)
 #define CONFFILE	"_mpoprc"
 #define UIDLSFILE	"_uidls"
-#define NETRCFILE	"_netrc"
+#define USERNETRCFILE	"_netrc"
+#define SYSNETRCFILE	"netrc"
 #else /* UNIX */
 #define CONFFILE	".mpoprc"
 #define UIDLSFILE	".mpop_uidls/%U_at_%H"
-#define NETRCFILE	".netrc"
+#define USERNETRCFILE	".netrc"
+#define SYSNETRCFILE	"netrc"
 #endif
 
 
@@ -234,24 +231,36 @@ char *mpop_sanitize_string(char *str)
 /*
  * mpop_password_callback()
  *
- * This function will be called by smtp_auth() to get a password if none was
- * given. It reads a password with getpass()
+ * This function will be called by pop3_auth() to get a password if none was
+ * given. It tries to read a password from .netrc. If that fails, it tries to 
+ * get it from the system's keychain (if available). If that fails, it reads a 
+ * password with getpass().
  * It must return NULL on failure or a password in an allocated buffer.
  */
 
 char *mpop_password_callback(const char *hostname, const char *user)
 {
-    char *homedir;
+    char *netrc_directory;
     char *netrc_filename;
     netrc_entry *netrc_hostlist;
     netrc_entry *netrc_host;
+#ifdef HAVE_GNOMEKEYRING
+    const char *protocol = "pop3";
+    GList *found_list = NULL;
+    GnomeKeyringNetworkPasswordData *found;
+#endif
+#ifdef HAVE_MACOSXKEYRING
+    void *password_data;
+    UInt32 password_length;
+    OSStatus status;
+#endif
     char *prompt;
     char *gpw;
     char *password = NULL;
 
-    homedir = get_homedir();
-    netrc_filename = get_filename(homedir, NETRCFILE);
-    free(homedir);
+    netrc_directory = get_homedir();
+    netrc_filename = get_filename(netrc_directory, USERNETRCFILE);
+    free(netrc_directory);
     if ((netrc_hostlist = parse_netrc(netrc_filename)))
     {
 	if ((netrc_host = search_netrc(netrc_hostlist, hostname, user)))
@@ -261,6 +270,67 @@ char *mpop_password_callback(const char *hostname, const char *user)
 	free_netrc_entry_list(netrc_hostlist);
     }
     free(netrc_filename);
+
+    if (!password)
+    {
+	netrc_directory = get_sysconfdir();
+	netrc_filename = get_filename(netrc_directory, SYSNETRCFILE);
+	free(netrc_directory);
+	if ((netrc_hostlist = parse_netrc(netrc_filename)))
+	{
+	    if ((netrc_host = search_netrc(netrc_hostlist, hostname, user)))
+	    {
+		password = xstrdup(netrc_host->password);
+	    }
+	    free_netrc_entry_list(netrc_hostlist);
+	}
+	free(netrc_filename);
+    }
+
+#ifdef HAVE_GNOMEKEYRING
+    if (!password)
+    {
+    	g_set_application_name(PACKAGE);
+    	if (gnome_keyring_find_network_password_sync(
+		    user,     /* user */
+		    NULL,     /* domain */
+		    hostname, /* server */
+		    NULL,     /* object */
+		    protocol, /* protocol */
+		    NULL,     /* authtype */
+		    0,        /* port */
+		    &found_list) == GNOME_KEYRING_RESULT_OK)
+	{
+	    found = (GnomeKeyringNetworkPasswordData *) found_list->data;
+	    if (found->password)
+		password = g_strdup(found->password);
+	}
+	gnome_keyring_network_password_list_free(found_list);
+    }
+#endif /* HAVE_GNOMEKEYRING */
+
+#ifdef HAVE_MACOSXKEYRING
+    if (!password)
+    {
+	if (SecKeychainFindInternetPassword(
+	    	    NULL,
+	    	    strlen(hostname), hostname,
+	    	    0, NULL,
+	    	    strlen(user), user,
+	    	    0, (char *)NULL,
+	    	    0,
+	    	    kSecProtocolTypeSMTP,
+	    	    kSecAuthenticationTypeDefault,
+	    	    &password_length, &password_data,
+	    	    NULL) == noErr)
+	{
+	    password = xmalloc((password_length + 1) * sizeof(char));
+	    strncpy(password, password_data, (size_t)password_length);
+	    password[password_length] = '\0';
+	    SecKeychainItemFreeContent(NULL, password_data);
+	}
+    }
+#endif /* HAVE_MACOSXKEYRING */
     
     if (!password)
     {
@@ -272,7 +342,7 @@ char *mpop_password_callback(const char *hostname, const char *user)
 	    password = xstrdup(gpw);
 	}
     }
-    
+
     return password;
 }
 			    
@@ -432,8 +502,9 @@ int mpop_serverinfo(account_t *acc, int debug, char **errmsg, char **errstr)
     {
 	tci = tls_cert_info_new();
 	if ((e = pop3_tls_init(session, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
-		!= TLS_EOK)
+			acc->tls_trust_file, acc->tls_crl_file,
+			acc->tls_force_sslv3, acc->tls_min_dh_prime_bits,
+			acc->tls_priorities, errstr)) != TLS_EOK)
 	{
 	    pop3_session_free(session);
 	    e = exitcode_tls(e);
@@ -840,13 +911,11 @@ void mpop_retr_progress_abort(long i UNUSED, long number UNUSED,
  * to one, for example by a signal handler.
  */
 
-#if HAVE_SIGACTION
 volatile sig_atomic_t mpop_retrmail_abort = 0;
 void mpop_retrmail_signal_handler(int signum UNUSED)
 {   
     mpop_retrmail_abort = 1;
 }
-#endif
 
 int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	account_t *acc, int debug, 
@@ -863,7 +932,6 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     FILE *uidls_fileptr;
     list_t *uidl_list;
     uidl_t *uidl;
-    int cmp;
     /* for status output: */
     char *sizestr;
     /* For errors that happen after a message is retrieved and delivered, and
@@ -893,8 +961,9 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     if (acc->tls)
     {
 	if ((e = pop3_tls_init(session, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
-		!= TLS_EOK)
+			acc->tls_trust_file, acc->tls_crl_file,
+			acc->tls_force_sslv3, acc->tls_min_dh_prime_bits,
+			acc->tls_priorities, errstr)) != TLS_EOK)
 	{
 	    pop3_session_free(session);
 	    return exitcode_tls(e);
@@ -988,14 +1057,12 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     }
 #endif /* HAVE_TLS */
     
-#if HAVE_SIGACTION
     if (mpop_retrmail_abort)
     {
 	mpop_endsession(session, 0);
 	*errstr = xasprintf(_("operation aborted"));
 	return EX_TEMPFAIL;
     }
-#endif
 
     /* authenticate */
     if ((e = pop3_auth(session, acc->auth_mech, acc->username, 
@@ -1043,14 +1110,12 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	}
     }
     
-#if HAVE_SIGACTION
     if (mpop_retrmail_abort)
     {
 	mpop_endsession(session, 0);
 	*errstr = xasprintf(_("operation aborted"));
 	return EX_TEMPFAIL;
     }
-#endif
 
     /* get status and scan listing */
     if ((e = pop3_stat(session, errmsg, errstr)) != POP3_EOK)
@@ -1060,51 +1125,20 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     }
     if (session->total_number > 0)
     {
-	if ((e = pop3_list(session, 
-#if HAVE_SIGACTION
-			&mpop_retrmail_abort, 
-#endif
-			errmsg, errstr)) != POP3_EOK)
+	if ((e = pop3_list(session, &mpop_retrmail_abort, errmsg, errstr))
+		!= POP3_EOK)
 	{
 	    mpop_endsession(session, 0);
 	    return exitcode_pop3(e);
 	}
     }
     
-#if HAVE_SIGACTION
     if (mpop_retrmail_abort)
     {
 	mpop_endsession(session, 0);
 	*errstr = xasprintf(_("operation aborted"));
 	return EX_TEMPFAIL;
     }
-#endif
-
-    /* retrieve the UIDs */
-    if (session->total_number > 0)
-    {
-	if ((session->cap.flags & POP3_CAP_CAPA) 
-		&& !(session->cap.flags & POP3_CAP_UIDL))
-	{
-	    /* the POP3 server does not support UIDL */
-	}
-	else if ((e = pop3_uidl(session,
-#if HAVE_SIGACTION
-			&mpop_retrmail_abort, 
-#endif
-			errmsg, errstr)) != POP3_EOK)
-	{
-	    if (e != POP3_EUNAVAIL)
-	    {
-		mpop_endsession(session, 0);
-		return exitcode_pop3(e);
-	    }
-	    else
-	    {
-		/* the POP3 server does not support UIDL */
-	    }
-	}
-    }
 
     /* Load the list of UID lists (if the file does not exist, the returned
      * list will be empty) */
@@ -1122,54 +1156,29 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	list_insert(uidl_list, uidl);
     }
 
-    /* Identify new messages. Both the list of current UIDs from the POP3
-     * server (accessed with session->msg_uid[session_uids_sorted[i]]) and the
-     * list of already seen UIDs (accessed with uidl->uidv[j]) are sorted. */
+    /* retrieve the UIDs */
     if (session->total_number > 0)
     {
-	session->new_number = 0;
-	session->new_size = 0;
-	j = 0;
-	i = 0;
-	while (i < session->total_number)
+	if ((session->cap.flags & POP3_CAP_CAPA) 
+		&& !(session->cap.flags & POP3_CAP_UIDL))
 	{
-	    if (j < uidl->n)
+	    /* the POP3 server does not support UIDL */
+	}
+	else if ((e = pop3_uidl(session, uidl->uidv, uidl->n, acc->only_new,
+			&mpop_retrmail_abort, errmsg, errstr)) != POP3_EOK)
+	{
+	    if (e != POP3_EUNAVAIL)
 	    {
-		cmp = strcmp(uidl->uidv[j], 
-			session->msg_uid[session->uids_sorted[i]]);
+		mpop_endsession(session, 0);
+		return exitcode_pop3(e);
 	    }
 	    else
 	    {
-		cmp = +1;
-	    }
-	    if (cmp < 0)
-	    {
-		j++;
-	    }
-	    else if (cmp > 0)
-	    {
-		session->new_number++;
-		session->new_size += 
-		    session->msg_size[session->uids_sorted[i]];
-		i++;
-	    }
-	    else
-	    {
-		/* Set action to DELETE if we should retrieve only new messages.
-		 * Else leave it as NORMAL. */
-		if (acc->only_new)
-		{
-		    session->msg_action[session->uids_sorted[i]] = 
-			POP3_MSG_ACTION_DELETE;
-		}
-		session->is_old[session->uids_sorted[i]] = 1;
-		session->old_number++;
-		i++;
-		j++;
+		/* the POP3 server does not support UIDL */
 	    }
 	}
     }
-    
+
     /* Print status */
     if (print_status)
     {
@@ -1241,6 +1250,11 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	    if (acc->killsize >= 0 && session->msg_size[i - 1] >= acc->killsize)
 	    {
 		session->msg_action[i - 1] = POP3_MSG_ACTION_DELETE;
+		if (!(session->is_old[i - 1]))
+		{
+		    session->is_old[i - 1] = 1;
+       		    session->old_number++;
+		}
 		if (print_progress)
 		{
 		    if (acc->keep)
@@ -1269,7 +1283,6 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	}
     }
 
-#if HAVE_SIGACTION
     if (mpop_retrmail_abort)
     {
 	if (uidl_list)
@@ -1281,7 +1294,6 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	*errstr = xasprintf(_("operation aborted"));
 	return EX_TEMPFAIL;
     }
-#endif
 
     /* Header filtering */
     if (session->total_number > 0 && acc->filter)
@@ -1299,11 +1311,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
 	    mpop_endsession(session, 1);
 	    return EX_UNAVAILABLE;
 	}
-	if ((e = pop3_filter(session, 
-#if HAVE_SIGACTION
-			&mpop_retrmail_abort, 
-#endif
-			acc->filter, 
+	if ((e = pop3_filter(session, &mpop_retrmail_abort, acc->filter, 
 			print_progress ? mpop_filter_output : NULL, acc,
 			errmsg, errstr)) != POP3_EOK)
 	{
@@ -1327,10 +1335,7 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
     /* Retrieve mails and update the UIDL accordingly */
     if (session->total_number > 0)
     {
-	late_error = pop3_retr(session, 
-#if HAVE_SIGACTION
-		&mpop_retrmail_abort,
-#endif
+	late_error = pop3_retr(session, &mpop_retrmail_abort,
 		acc->delivery_method, acc->delivery_args,
 		print_progress ? mpop_retr_progress_start : NULL,
 		print_progress ? mpop_retr_progress : NULL,
@@ -1363,11 +1368,8 @@ int mpop_retrmail(const char *canonical_hostname, const char *local_user,
      * update the UIDL state and save it. */
     if (late_error == POP3_EOK && !acc->keep)
     {
-	if ((e = pop3_dele(session, 
-#if HAVE_SIGACTION
-		&mpop_retrmail_abort,
-#endif
-		errmsg, errstr)) != POP3_EOK)
+	if ((e = pop3_dele(session, &mpop_retrmail_abort, errmsg, errstr))
+		!= POP3_EOK)
 	{
 	    (void)uidls_write(acc->uidls_file, uidls_fileptr, uidl_list, NULL);
 	    list_xfree(uidl_list, uidl_free);
@@ -1578,26 +1580,29 @@ int make_needed_dirs(const char *pathname)
  */
 
 /* long options without a corresponding short option */
-#define LONGONLYOPT_VERSION 		0
-#define LONGONLYOPT_HELP    		1
-#define LONGONLYOPT_HOST		2
-#define LONGONLYOPT_PORT		3
-#define LONGONLYOPT_TIMEOUT		4
-#define LONGONLYOPT_PIPELINING		5
-#define LONGONLYOPT_AUTH		6
-#define LONGONLYOPT_USER		7
-#define LONGONLYOPT_TLS			8
-#define LONGONLYOPT_TLS_STARTTLS	9
-#define LONGONLYOPT_TLS_TRUST_FILE	10
-#define LONGONLYOPT_TLS_KEY_FILE	11
-#define LONGONLYOPT_TLS_CERT_FILE	12
-#define LONGONLYOPT_TLS_CERTCHECK	13
-#define LONGONLYOPT_TLS_FORCE_SSLV3	14
-#define LONGONLYOPT_KILLSIZE		15
-#define LONGONLYOPT_SKIPSIZE		16
-#define LONGONLYOPT_FILTER		17
-#define LONGONLYOPT_DELIVERY		18
-#define LONGONLYOPT_UIDLS_FILE		19
+#define LONGONLYOPT_VERSION 			0
+#define LONGONLYOPT_HELP    			1
+#define LONGONLYOPT_HOST			2
+#define LONGONLYOPT_PORT			3
+#define LONGONLYOPT_TIMEOUT			4
+#define LONGONLYOPT_PIPELINING			5
+#define LONGONLYOPT_AUTH			6
+#define LONGONLYOPT_USER			7
+#define LONGONLYOPT_TLS				8
+#define LONGONLYOPT_TLS_STARTTLS		9
+#define LONGONLYOPT_TLS_TRUST_FILE		10
+#define LONGONLYOPT_TLS_CRL_FILE		11
+#define LONGONLYOPT_TLS_KEY_FILE		12
+#define LONGONLYOPT_TLS_CERT_FILE		13
+#define LONGONLYOPT_TLS_CERTCHECK		14
+#define LONGONLYOPT_TLS_FORCE_SSLV3		15
+#define LONGONLYOPT_TLS_MIN_DH_PRIME_BITS	16
+#define LONGONLYOPT_TLS_PRIORITIES		17
+#define LONGONLYOPT_KILLSIZE			18
+#define LONGONLYOPT_SKIPSIZE			19
+#define LONGONLYOPT_FILTER			20
+#define LONGONLYOPT_DELIVERY			21
+#define LONGONLYOPT_UIDLS_FILE			22
 
 int main(int argc, char *argv[])
 {
@@ -1636,12 +1641,10 @@ int main(int argc, char *argv[])
     char *errstr;
     char *errmsg;
     /* signal handling */
-#if HAVE_SIGACTION
     struct sigaction signal_handler;
     struct sigaction old_sigterm_handler;
     struct sigaction old_sighup_handler;
     struct sigaction old_sigint_handler;
-#endif
     /* misc */
     struct servent *se;	    
     int c;
@@ -1652,45 +1655,56 @@ int main(int argc, char *argv[])
     /* option handling */
     struct option options[] =
     {
-	{ "version",         no_argument,       0, LONGONLYOPT_VERSION },
-	{ "help",            no_argument,       0, LONGONLYOPT_HELP },
-	{ "quiet",           no_argument,       0, 'q' },
-	{ "half-quiet",      no_argument,       0, 'Q' },
- 	{ "pretend",         no_argument,       0, 'P' },
-  	{ "debug",           no_argument,       0, 'd' },
-	{ "serverinfo",      no_argument,       0, 'S' },
-	{ "file",            required_argument, 0, 'C' },
-	{ "auth-only",       no_argument,       0, 'A' },
-	{ "all-accounts",    no_argument,       0, 'a' },
-	{ "status-only",     no_argument,       0, 's' }, 
-	{ "delivery",        required_argument, 0, LONGONLYOPT_DELIVERY },
-	{ "uidls-file",      required_argument, 0, LONGONLYOPT_UIDLS_FILE },
-	{ "only-new",        optional_argument, 0, 'n' },
-	{ "keep",            optional_argument, 0, 'k' },
-	{ "killsize",        required_argument, 0, LONGONLYOPT_KILLSIZE },
-	{ "skipsize",        required_argument, 0, LONGONLYOPT_SKIPSIZE },
-	{ "filter",          required_argument, 0, LONGONLYOPT_FILTER },
-	{ "host",            required_argument, 0, LONGONLYOPT_HOST },
-	{ "port",            required_argument, 0, LONGONLYOPT_PORT },
-	{ "timeout",         required_argument, 0, LONGONLYOPT_TIMEOUT},
-	{ "pipelining",      required_argument, 0, LONGONLYOPT_PIPELINING },
-	{ "auth",            optional_argument, 0, LONGONLYOPT_AUTH },
-	{ "user",            required_argument, 0, LONGONLYOPT_USER },
-	{ "tls",             optional_argument, 0, LONGONLYOPT_TLS },
-	{ "tls-starttls",    optional_argument, 0, LONGONLYOPT_TLS_STARTTLS },
-	{ "tls-trust-file",  required_argument, 0, LONGONLYOPT_TLS_TRUST_FILE },
-	{ "tls-key-file",    required_argument, 0, LONGONLYOPT_TLS_KEY_FILE },
-	{ "tls-cert-file",   required_argument, 0, LONGONLYOPT_TLS_CERT_FILE },
-	{ "tls-certcheck",   optional_argument, 0, LONGONLYOPT_TLS_CERTCHECK },
-	{ "tls-force-sslv3", optional_argument, 0, 
+	{ "version",               no_argument,       0, LONGONLYOPT_VERSION },
+	{ "help",                  no_argument,       0, LONGONLYOPT_HELP },
+	{ "quiet",                 no_argument,       0, 'q' },
+	{ "half-quiet",            no_argument,       0, 'Q' },
+ 	{ "pretend",               no_argument,       0, 'P' },
+  	{ "debug",                 no_argument,       0, 'd' },
+	{ "serverinfo",            no_argument,       0, 'S' },
+	{ "file",                  required_argument, 0, 'C' },
+	{ "auth-only",             no_argument,       0, 'A' },
+	{ "all-accounts",          no_argument,       0, 'a' },
+	{ "status-only",           no_argument,       0, 's' }, 
+	{ "delivery",              required_argument, 0, LONGONLYOPT_DELIVERY },
+	{ "uidls-file",            required_argument, 0, 
+	    LONGONLYOPT_UIDLS_FILE },
+	{ "only-new",              optional_argument, 0, 'n' },
+	{ "keep",                  optional_argument, 0, 'k' },
+	{ "killsize",              required_argument, 0, LONGONLYOPT_KILLSIZE },
+	{ "skipsize",              required_argument, 0, LONGONLYOPT_SKIPSIZE },
+	{ "filter",                required_argument, 0, LONGONLYOPT_FILTER },
+	{ "host",                  required_argument, 0, LONGONLYOPT_HOST },
+	{ "port",                  required_argument, 0, LONGONLYOPT_PORT },
+	{ "timeout",               required_argument, 0, LONGONLYOPT_TIMEOUT},
+	{ "pipelining",            required_argument, 0, 
+	    LONGONLYOPT_PIPELINING },
+	{ "auth",                  optional_argument, 0, LONGONLYOPT_AUTH },
+	{ "user",                  required_argument, 0, LONGONLYOPT_USER },
+	{ "tls",                   optional_argument, 0, LONGONLYOPT_TLS },
+	{ "tls-starttls",          optional_argument, 0, 
+	    LONGONLYOPT_TLS_STARTTLS },
+	{ "tls-trust-file",        required_argument, 0, 
+	    LONGONLYOPT_TLS_TRUST_FILE },
+	{ "tls-crl-file",          required_argument, 0, 
+	    LONGONLYOPT_TLS_CRL_FILE },
+	{ "tls-key-file",          required_argument, 0, 
+	    LONGONLYOPT_TLS_KEY_FILE },
+	{ "tls-cert-file",         required_argument, 0, 
+	    LONGONLYOPT_TLS_CERT_FILE },
+	{ "tls-certcheck",         optional_argument, 0, 
+	    LONGONLYOPT_TLS_CERTCHECK },
+	{ "tls-force-sslv3",       optional_argument, 0, 
 	    LONGONLYOPT_TLS_FORCE_SSLV3 },
+	{ "tls-min-dh-prime-bits", required_argument, 0, 
+	    LONGONLYOPT_TLS_MIN_DH_PRIME_BITS },
+	{ "tls-priorities",        required_argument, 0, 
+	    LONGONLYOPT_TLS_PRIORITIES },
 	{ 0, 0, 0, 0 }
     };
     
     /* Avoid the side effects of text mode interpretations on DOS systems. */
-#ifdef W32_NATIVE
-    _fmode = _O_BINARY;
-#elif defined DJGPP
+#if defined W32_NATIVE || defined DJGPP
     _fmode = O_BINARY;
 #endif
 
@@ -1913,6 +1927,13 @@ int main(int argc, char *argv[])
 		cmdline_account->mask |= ACC_TLS_TRUST_FILE;
 		break;
 
+	    case LONGONLYOPT_TLS_CRL_FILE:
+		free(cmdline_account->tls_crl_file);
+		cmdline_account->tls_crl_file = (*optarg == '\0')
+  		    ? NULL : expand_tilde(optarg);
+		cmdline_account->mask |= ACC_TLS_CRL_FILE;
+		break;
+
 	    case LONGONLYOPT_TLS_KEY_FILE:
 		free(cmdline_account->tls_key_file);
 		cmdline_account->tls_key_file = (*optarg == '\0')
@@ -1962,6 +1983,38 @@ int main(int argc, char *argv[])
 		}
 		cmdline_account->mask |= ACC_TLS_FORCE_SSLV3;
 		break;
+		
+	    case LONGONLYOPT_TLS_MIN_DH_PRIME_BITS:
+		if (*optarg == '\0')
+		{
+		    cmdline_account->tls_min_dh_prime_bits = -1;
+		}
+		else
+		{
+		    cmdline_account->tls_min_dh_prime_bits = 
+			get_non_neg_int(optarg);
+		    if (cmdline_account->tls_min_dh_prime_bits < 1)
+		    {
+			print_error(_("invalid argument %s for %s"), 
+			     	optarg, "--tls-min-dh-prime-bits");
+			error_code = 1;
+		    }
+		}
+		cmdline_account->mask |= ACC_TLS_MIN_DH_PRIME_BITS;
+		break;
+
+	    case LONGONLYOPT_TLS_PRIORITIES:
+		free(cmdline_account->tls_priorities);
+		if (*optarg)
+		{
+		    cmdline_account->tls_priorities = xstrdup(optarg);
+		}
+		else
+		{
+		    cmdline_account->tls_priorities = NULL;
+		}
+		cmdline_account->mask |= ACC_TLS_PRIORITIES;
+		break;
 
 	    case 'n':
 	    	if (!optarg || is_on(optarg))
@@ -2163,9 +2216,21 @@ int main(int argc, char *argv[])
 	printf(_(", LOCALEDIR is %s"), LOCALEDIR);
 #else
 	printf(_("disabled"));
+#endif
+	printf("\n");
+	printf(_("Keyring support: "));
+#if !defined HAVE_GNOMEKEYRING && !defined HAVE_MACOSXKEYRING
+	printf(_("none"));
+#else
+# ifdef HAVE_GNOMEKEYRING
+	printf(_("Gnome "));
+# endif
+# ifdef HAVE_MACOSXKEYRING
+	printf(_("MacOS "));
+# endif
 #endif
 	printf("\n\n");
-	printf(_("Copyright (C) 2007 Martin Lambers and others.\n"
+	printf(_("Copyright (C) 2009 Martin Lambers and others.\n"
 		    "This is free software.  You may redistribute copies of "
 		        "it under the terms of\n"
 		    "the GNU General Public License "
@@ -2213,6 +2278,8 @@ int main(int argc, char *argv[])
 			"encryption.\n"
 		"  --tls-starttls[=(on|off)]    Enable/disable STLS for TLS.\n"
 		"  --tls-trust-file=[file]      Set/unset trust file for TLS.\n"
+		"  --tls-crl-file=[file]        Set/unset revocation file for "
+			"TLS.\n"
 	        "  --tls-key-file=[file]        Set/unset private key file for "
 			"TLS.\n"
 		"  --tls-cert-file=[file]       Set/unset private cert file "
@@ -2221,6 +2288,9 @@ int main(int argc, char *argv[])
 			"certificate checks for TLS.\n"
 		"  --tls-force-sslv3[=(on|off)] Enable/disable restriction to "
 			"SSLv3.\n"
+		"  --tls-min-dh-prime-bits=[b]  Set/unset minimum bit size of "
+			"DH prime.\n"
+	    	"  --tls-priorities=[prios]     Set/unset TLS priorities.\n"
 	        "Options specific to mail retrieval mode:\n"
 		"  -q, --quiet                  Do not display status or "
 			"progress information.\n"
@@ -2382,11 +2452,11 @@ int main(int argc, char *argv[])
 		printf(_("using account %s from %s\n"), 
 			account->id, account->conffile);
 	    }
-	    printf("host            = %s\n"
-		    "port            = %d\n",
+	    printf("host                  = %s\n"
+		    "port                  = %d\n",
 		    account->host,
 		    account->port);
-	    printf("timeout         = ");
+	    printf("timeout               = ");
 	    if (account->timeout <= 0)
 	    {
 		printf(_("off\n"));
@@ -2402,10 +2472,10 @@ int main(int argc, char *argv[])
 		    printf(_("%d seconds\n"), account->timeout);
 		}
 	    }
-	    printf("pipelining      = %s\n", 
+	    printf("pipelining            = %s\n", 
 		    account->pipelining == 0 ? _("off") 
 		    : account->pipelining == 1 ? _("on") : _("auto"));
-	    printf("auth            = ");
+	    printf("auth                  = ");
 	    if (account->auth_mech[0] == '\0')
 	    {
 		printf(_("choose\n"));
@@ -2414,16 +2484,17 @@ int main(int argc, char *argv[])
 	    {
 		printf("%s\n", account->auth_mech);
 	    }
-	    printf("user            = %s\n"
-		    "password        = %s\n"
-		    "ntlmdomain      = %s\n"
-		    "tls             = %s\n"
-		    "tls_starttls    = %s\n"
-		    "tls_trust_file  = %s\n"
-		    "tls_key_file    = %s\n"
-		    "tls_cert_file   = %s\n"
-		    "tls_certcheck   = %s\n"
-		    "tls_force_sslv3 = %s\n",
+	    printf("user                  = %s\n"
+		    "password              = %s\n"
+		    "ntlmdomain            = %s\n"
+		    "tls                   = %s\n"
+		    "tls_starttls          = %s\n"
+		    "tls_trust_file        = %s\n"
+		    "tls_crl_file          = %s\n"
+		    "tls_key_file          = %s\n"
+		    "tls_cert_file         = %s\n"
+		    "tls_certcheck         = %s\n"
+		    "tls_force_sslv3       = %s\n",
 		    account->username ? account->username : _("(not set)"),
 		    account->password ? "*" : _("(not set)"),
 		    account->ntlmdomain ? account->ntlmdomain : _("(not set)"),
@@ -2431,15 +2502,28 @@ int main(int argc, char *argv[])
 		    account->tls_nostarttls ? _("off") : _("on"),
 		    account->tls_trust_file ? 
 		    	account->tls_trust_file : _("(not set)"),
+		    account->tls_crl_file ? 
+		    	account->tls_crl_file : _("(not set)"),
 		    account->tls_key_file ? 
 		    	account->tls_key_file : _("(not set)"),
 		    account->tls_cert_file ? 
 		    	account->tls_cert_file : _("(not set)"),
 		    account->tls_nocertcheck ? _("off") : _("on"),
 		    account->tls_force_sslv3 ? _("on") : _("off"));
+	    printf("tls_min_dh_prime_bits = ");
+	    if (account->tls_min_dh_prime_bits >= 0)
+	    {
+		printf("%d\n", account->tls_min_dh_prime_bits);
+	    }
+	    else
+	    {
+		printf("%s\n", _("(not set)"));
+	    }
+	    printf("tls_priorities        = %s\n", account->tls_priorities 
+		    ? account->tls_priorities : _("(not set)"));
 	    if (retrmail)
 	    {
-		printf("delivery        = ");
+		printf("delivery              = ");
 		if (account->delivery_method == DELIVERY_METHOD_MDA)
 		{
 		    printf("mda");
@@ -2453,13 +2537,13 @@ int main(int argc, char *argv[])
 		    printf("mbox");
 		}
 		printf(" %s\n", account->delivery_args);
-		printf("uidls file      = %s\n"
-			"only_new        = %s\n"
-			"keep            = %s\n", 
+		printf("uidls file            = %s\n"
+			"only_new              = %s\n"
+			"keep                  = %s\n", 
 			account->uidls_file,
 			account->only_new ? _("on") : _("off"),
 			account->keep ? _("on") : _("off"));
-		printf("killsize        = ");
+		printf("killsize              = ");
 		if (account->killsize < 0)
 		{
 		    printf(_("off\n"));
@@ -2468,7 +2552,7 @@ int main(int argc, char *argv[])
 		{
 		    printf("%ld\n", account->killsize);
 		}
-		printf("skipsize        = ");
+		printf("skipsize              = ");
 		if (account->skipsize < 0)
 		{
 		    printf(_("off\n"));
@@ -2477,7 +2561,7 @@ int main(int argc, char *argv[])
 		{
 		    printf("%ld\n", account->skipsize);
 		}
-		printf("filter          = %s\n",
+		printf("filter                = %s\n",
 			account->filter ? account->filter : _("(not set)"));
 	    }
 	}
@@ -2570,15 +2654,15 @@ int main(int argc, char *argv[])
 	    {
 		local_user = get_username();
 	    }
-#if HAVE_SIGACTION
 	    mpop_retrmail_abort = 0;
     	    signal_handler.sa_handler = mpop_retrmail_signal_handler;
 	    sigemptyset(&signal_handler.sa_mask);
     	    signal_handler.sa_flags = 0;
     	    (void)sigaction(SIGTERM, &signal_handler, &old_sigterm_handler);
+#ifdef SIGHUP
 	    (void)sigaction(SIGHUP, &signal_handler, &old_sighup_handler);
-	    (void)sigaction(SIGINT, &signal_handler, &old_sigint_handler);
 #endif
+	    (void)sigaction(SIGINT, &signal_handler, &old_sigint_handler);
 	    if ((error_code = mpop_retrmail(canonical_hostname, local_user,
 			    account, debug, print_status, print_progress, 
 			    auth_only, status_only, &errmsg, &errstr)) != EX_OK)
@@ -2607,15 +2691,15 @@ int main(int argc, char *argv[])
 		    print_error(_("error during mail retrieval"));
 		}
 	    }
-#if HAVE_SIGACTION
     	    (void)sigaction(SIGTERM, &old_sigterm_handler, NULL);
+#ifdef SIGHUP
 	    (void)sigaction(SIGHUP, &old_sighup_handler, NULL);
+#endif
 	    (void)sigaction(SIGINT, &old_sigint_handler, NULL);
 	    if (mpop_retrmail_abort)
 	    {
 		break;
 	    }
-#endif
 	}
     }
     free(canonical_hostname);
