@@ -3,7 +3,7 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,8 @@
 #ifndef NET_H
 #define NET_H
 
+#include "readbuf.h"
+
 
 /*
  * If a function with an 'errstr' argument returns a value != NET_EOK,
@@ -37,15 +39,6 @@
 #define NET_ECONNECT		4	/* Cannot connect */
 #define NET_EIO			5	/* Input/output error */
 
-/* This structure is used as a read buffer for net_gets. Do not access it
- * directly. Use net_readbuf_init() to initialize it. */
-typedef struct
-{
-    int count;
-    char *ptr;
-    char buf[4096];
-} net_readbuf_t;
-
 /*
  * net_lib_init()
  *
@@ -71,20 +64,13 @@ int net_lib_init(char **errstr);
  * address of the server will be stored in '*address', or NULL if this 
  * information is not available.
  * The strings must be deallocated when not used anymore.
- * The file descriptor is returned in 'fd'.
+ * The file descriptor is returned in 'fd'. It can be closed with close().
  *
  * Used error codes: NET_EHOSTNOTFOUND, NET_ESOCKET, NET_ECONNECT
  */
 int net_open_socket(const char *hostname, int port, int timeout, int *fd, 
 	char **canonical_name, char **address, char **errstr);
 
-/*
- * net_readbuf_init()
- *
- * Initialize a net_readbuf_t for first use.
- */
-void net_readbuf_init(net_readbuf_t *readbuf);
-
 /*
  * net_gets()
  * 
@@ -97,7 +83,7 @@ void net_readbuf_init(net_readbuf_t *readbuf);
  * all read operations on 'fd'.
  * Used error codes: NET_EIO
  */
-int net_gets(int fd, net_readbuf_t *readbuf,
+int net_gets(int fd, readbuf_t *readbuf,
 	char *str, size_t size, size_t *len, char **errstr);
 
 /*
@@ -108,13 +94,6 @@ int net_gets(int fd, net_readbuf_t *readbuf,
  */
 int net_puts(int fd, const char *s, size_t len, char **errstr);
 
-/*
- * net_close_socket()
- *
- * Closes a socket.
- */
-void net_close_socket(int fd);
-
 /*
  * net_get_canonical_hostname()
  *
