@@ -3,8 +3,9 @@
  *
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
  * Martin Lambers <marlam@marlam.de>
+ * Dimitrios Apostolou <jimis@gmx.net> (UID handling)
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -33,9 +34,8 @@
 #include <strings.h>
 #include <ctype.h>
 #include <errno.h>
-#if HAVE_SIGACTION
-# include <signal.h>
-#endif
+#include <signal.h>
+#include <unistd.h>
 
 #ifdef HAVE_LIBIDN
 # include <idna.h>
@@ -55,6 +55,7 @@
 #include "xvasprintf.h"
 
 #include "delivery.h"
+#include "readbuf.h"
 #include "net.h"
 #include "tools.h"
 #include "stream.h"
@@ -62,6 +63,7 @@
 # include "tls.h"
 #endif /* HAVE_TLS */
 #include "pop3.h"
+#include "uidls.h"
 
 
 /*
@@ -128,7 +130,7 @@ pop3_session_t *pop3_session_new(int pipelining,
     session->server_hostname = NULL;
     session->server_canonical_name = NULL;
     session->server_address = NULL;
-    net_readbuf_init(&(session->readbuf));
+    readbuf_init(&(session->readbuf));
 #ifdef HAVE_TLS
     tls_clear(&session->tls);
 #endif /* HAVE_TLS */
@@ -148,7 +150,6 @@ pop3_session_t *pop3_session_new(int pipelining,
     session->new_size = 0;
     session->msg_size = NULL;
     session->msg_uid = NULL;
-    session->uids_sorted = NULL;
     session->msg_action = NULL;
 
     return session;
@@ -180,7 +181,6 @@ void pop3_session_free(pop3_session_t *session)
 	}
 	free(session->msg_uid);
     }
-    free(session->uids_sorted);
     free(session->msg_action);
     free(session->is_old);
     free(session->msg_size);
@@ -235,8 +235,8 @@ int pop3_gets(pop3_session_t *session, size_t *len, char **errstr)
 #ifdef HAVE_TLS
     if (tls_is_active(&session->tls))
     {
-	e = (tls_gets(&session->tls, session->buffer, POP3_BUFSIZE, len, errstr)
-		!= TLS_EOK);
+	e = (tls_gets(&session->tls, &(session->readbuf),
+		    session->buffer, POP3_BUFSIZE, len, errstr) != TLS_EOK);
     }
     else
     {
@@ -755,7 +755,8 @@ int pop3_capa(pop3_session_t *session, char **errstr)
 	    /* We know that the line ends with "\n" */
 	    while (!isspace((unsigned char)session->buffer[i]))
 	    {
-		session->buffer[i] = toupper((unsigned char)session->buffer[i]);
+		session->buffer[i] = 
+		    c_toupper((unsigned char)session->buffer[i]);
 		i++;
 	    }
 	    if (strncmp(session->buffer, "TOP", 3) == 0)
@@ -781,7 +782,7 @@ int pop3_capa(pop3_session_t *session, char **errstr)
 		while (session->buffer[i])
 		{
 		    session->buffer[i] = 
-			toupper((unsigned char)session->buffer[i]);
+			c_toupper((unsigned char)session->buffer[i]);
 		    i++;
 		}
 		if (strstr(session->buffer + 6, "NEVER"))
@@ -814,7 +815,7 @@ int pop3_capa(pop3_session_t *session, char **errstr)
 		while (session->buffer[i])
 		{
 		    session->buffer[i] = 
-			toupper((unsigned char)session->buffer[i]);
+			c_toupper((unsigned char)session->buffer[i]);
 		    i++;
 		}
 		if (strstr(session->buffer + 5, "PLAIN"))
@@ -904,12 +905,14 @@ int pop3_capa(pop3_session_t *session, char **errstr)
  */
 
 #ifdef HAVE_TLS
-int pop3_tls_init(pop3_session_t *session, const char *tls_key_file, 
-	const char *tls_ca_file, const char *tls_trust_file, 
-	int force_sslv3, char **errstr)
+int pop3_tls_init(pop3_session_t *session,
+	const char *tls_key_file, const char *tls_cert_file, 
+	const char *tls_trust_file, const char *tls_crl_file,
+	int force_sslv3, int min_dh_prime_bits, const char *priorities,
+	char **errstr)
 {
-    return tls_init(&session->tls, tls_key_file, tls_ca_file, tls_trust_file, 
-	    force_sslv3, errstr);
+    return tls_init(&session->tls, tls_key_file, tls_cert_file, tls_trust_file,
+	    tls_crl_file, force_sslv3, min_dh_prime_bits, priorities, errstr);
 }
 #endif /* HAVE_TLS */
 
@@ -939,6 +942,12 @@ int pop3_tls_stls(pop3_session_t *session, char **errmsg, char **errstr)
 	*errstr = xasprintf(_("command %s failed"), "STLS");
 	return POP3_EPROTO;
     }
+    if (!readbuf_is_empty(&(session->readbuf)))
+    {
+	*errmsg = NULL;
+	*errstr = xasprintf(_("command %s failed"), "STLS");
+	return POP3_EPROTO;
+    }
     return POP3_EOK;
 }
 #endif /* HAVE_TLS */
@@ -1034,34 +1043,6 @@ int pop3_stat(pop3_session_t *session, char **errmsg, char **errstr)
  * see pop3.h
  */
 
-/* Helper: insert the index of an uid such that the array of indices gives the
- * UIDs in sorted (ascending) order */
-void insert_sorted(long *strings_sorted, long len, char **strings, 
-	long new_index)
-{
-    long a = 0;
-    long b = len - 1;
-    long c;
-
-    while (b >= a)
-    {
-	c = (a + b) / 2;
-	if (strcmp(strings[strings_sorted[c]], strings[new_index]) <= 0)
-	{
-	    a = c + 1;
-	}
-	else
-	{
-	    b = c - 1;
-	}
-    }
-    for (c = len; c > a; c--)
-    {
-	strings_sorted[c] = strings_sorted[c - 1];
-    }
-    strings_sorted[a] = new_index;
-}
-
 /* Helper: Check if a UID is valid. */
 int pop3_uidl_check_uid(const char *uid)
 {
@@ -1083,11 +1064,8 @@ int pop3_uidl_check_uid(const char *uid)
     return (p != uid);
 }
 
-int pop3_uidl(pop3_session_t *session, 
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
-	char **errmsg, char **errstr)
+int pop3_uidl(pop3_session_t *session, char **uidv, long uidv_len, int only_new,
+	volatile sig_atomic_t *abort, char **errmsg, char **errstr)
 {
     int e;
     long i;
@@ -1117,20 +1095,18 @@ int pop3_uidl(pop3_session_t *session,
     {
 	session->msg_uid[i] = NULL;
     }
-    session->uids_sorted = (session->total_number == 0) ? NULL :
-	xmalloc(session->total_number * sizeof(long));
 
     /* get 'total_number' UIDs plus one stop line (".") */
+    session->new_number = 0;
+    session->new_size= 0;
     for (i = 0; i < session->total_number + 1; i++)
     {
-#if HAVE_SIGACTION
 	if (*abort)
 	{
 	    e = POP3_EABORT;
 	    *errstr = xasprintf(_("operation aborted"));
 	    goto error_exit;
 	}
-#endif
 	if ((e = pop3_gets(session, &l, errstr)) != POP3_EOK)
 	{
 	    goto error_exit;
@@ -1176,7 +1152,24 @@ int pop3_uidl(pop3_session_t *session,
 		goto invalid_reply;
 	    }
 	    session->msg_uid[n - 1] = xstrdup(p);
-	    insert_sorted(session->uids_sorted, i, session->msg_uid, n - 1);
+	    if (bsearch(&session->msg_uid[n - 1], uidv, uidv_len, sizeof(*uidv),
+			uidls_uidcmp) == NULL)
+	    {
+		/* The UID is new */
+		session->is_old[n - 1]= 0;
+		session->new_number++;
+		session->new_size += session->msg_size[n - 1];
+	    }
+	    else
+	    {
+		/* The UID is old */
+		if (only_new)
+		{
+		    session->msg_action[n - 1]= POP3_MSG_ACTION_DELETE;
+		}
+		session->is_old[n - 1] = 1;
+		session->old_number++;
+	    }
 	}
     }
     
@@ -1210,10 +1203,7 @@ error_exit:
  * see pop3.h
  */
 
-int pop3_list(pop3_session_t *session,
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_list(pop3_session_t *session, volatile sig_atomic_t *abort,
 	char **errmsg, char **errstr)
 {
     int e;
@@ -1249,13 +1239,11 @@ int pop3_list(pop3_session_t *session,
     /* get 'total_number' sizes plus one stop line (".") */
     for (i = 0; i < session->total_number + 1; i++)
     {
-#if HAVE_SIGACTION
 	if (*abort)
 	{
 	    *errstr = xasprintf(_("operation aborted"));
 	    return POP3_EABORT;
 	}
-#endif
 	if ((e = pop3_gets(session, &l, errstr)) != POP3_EOK)
 	{
 	    return e;
@@ -1328,10 +1316,7 @@ invalid_reply:
  * Used error codes: POP3_EIO, POP3_EPROTO
  */
 
-int pop3_pipe(pop3_session_t *session, 
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_pipe(pop3_session_t *session, volatile sig_atomic_t *abort,
 	FILE *tmpf, FILE *f, long i, 
 	int full_mail, int from_quoting,	
 	void (*progress_start)(long i, long number, long size),
@@ -1361,7 +1346,6 @@ int pop3_pipe(pop3_session_t *session,
     line_continues = 0;
     for (;;)
     {
-#if HAVE_SIGACTION
 	if (*abort)
 	{
 	    if (progress_abort)
@@ -1372,7 +1356,6 @@ int pop3_pipe(pop3_session_t *session,
 	    *errstr = xasprintf(_("operation aborted"));
 	    return POP3_EABORT;
 	}
-#endif
 	line_starts = !line_continues;
 	if (read_from_tmpf)
 	{
@@ -1508,7 +1491,7 @@ int pop3_pipe(pop3_session_t *session,
 	progress_end(i, session->total_number, session->msg_size[i - 1]);
     }
     if (full_mail && !session->count_newline_as_crlf 
-	    && rcvd < session->msg_size[i])
+	    && rcvd < session->msg_size[i - 1])
     {
 	session->count_newline_as_crlf = 1;
     }
@@ -1594,6 +1577,11 @@ int pop3_retr_get_from_addr(pop3_session_t *session, FILE *f, char **from_addr,
 		/* this is the blank line separating the header from the body */
 		break;
 	    }
+	    else if (l == 1 && session->buffer[0] == '.')
+	    {
+		/* this mail ends here; it has no body */
+		break;
+	    }
 	    else if (!*from_addr)
 	    {
 		if (strncasecmp(session->buffer, "Return-Path:", 12) == 0)
@@ -1739,10 +1727,7 @@ int pop3_write_received_header(pop3_session_t *session, FILE *f, char **errstr)
  * unsuable thereafter.
  */
 
-int pop3_delivery(pop3_session_t *session,
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_delivery(pop3_session_t *session, volatile sig_atomic_t *abort,
 	int filter, 
 	void (*filter_output)(long i, long number, int new_status, 
 	    void *filter_output_data),
@@ -1797,14 +1782,12 @@ int pop3_delivery(pop3_session_t *session,
 		    ? pipeline_min : 0);
 		recv_index++)
 	{
-#if HAVE_SIGACTION
 	    if (*abort)
 	    {
 		e = POP3_EABORT;
 		*errstr = xasprintf(_("operation aborted"));
 		goto error_exit;
 	    }
-#endif
 	    if (session->msg_action[recv_index] != POP3_MSG_ACTION_NORMAL)
 	    {
 		continue;
@@ -1844,7 +1827,7 @@ int pop3_delivery(pop3_session_t *session,
 		{
 		    goto error_exit;
 		}
-		if (fseek(tmpf, 0L, SEEK_SET) != 0)
+		if (fseeko(tmpf, 0, SEEK_SET) != 0)
 	    	{
 		    *errstr = xasprintf(_("cannot rewind temporary file: %s"), 
 			    strerror(errno));
@@ -1874,10 +1857,7 @@ int pop3_delivery(pop3_session_t *session,
 		}
 	    }
 	    /* pipe the message (headers) */
-	    if ((e = pop3_pipe(session, 
-#if HAVE_SIGACTION
-			    abort, 
-#endif
+	    if ((e = pop3_pipe(session, abort, 
 			    tmpf, delivery->pipe, 
 			    recv_index + 1, !filter, 
 			    delivery->need_from_quoting,
@@ -1903,6 +1883,11 @@ int pop3_delivery(pop3_session_t *session,
 		{
 		    /* kill mail */
 		    session->msg_action[recv_index] = POP3_MSG_ACTION_DELETE;
+		    if (!(session->is_old[recv_index]))
+		    {
+			session->is_old[recv_index] = 1;
+			session->old_number++;
+		    }
 		    if (filter_output)
 		    {
 			filter_output(recv_index + 1, session->total_number, 
@@ -1919,7 +1904,7 @@ int pop3_delivery(pop3_session_t *session,
 			     	POP3_MSG_ACTION_IGNORE, filter_output_data);
 		    }
 		}
-		else /* filter_exit_code == 3 */
+		else /* filter_exit_code unknown */
 		{
 		    e = POP3_EDELIVERY;
 		    goto error_exit;
@@ -2009,20 +1994,14 @@ error_exit:
  * see pop3.h
  */
 
-int pop3_filter(pop3_session_t *session,
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort, 
-#endif
+int pop3_filter(pop3_session_t *session, volatile sig_atomic_t *abort, 
 	const char *filtercmd,
 	void (*filter_output)(long i, long number, int new_action, 
 	    void *filter_output_data),
 	void *filter_output_data, 
 	char **errmsg, char **errstr)
 {
-    return pop3_delivery(session, 
-#if HAVE_SIGACTION
-	    abort, 
-#endif
+    return pop3_delivery(session, abort, 
 	    1, filter_output, filter_output_data,
 	    DELIVERY_METHOD_FILTER, filtercmd, 
 	    NULL, NULL, NULL, NULL, errmsg, errstr);
@@ -2035,10 +2014,7 @@ int pop3_filter(pop3_session_t *session,
  * see pop3.h
  */
 
-int pop3_retr(pop3_session_t *session,
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort, 
-#endif
+int pop3_retr(pop3_session_t *session, volatile sig_atomic_t *abort, 
 	int delivery_method, const char *delivery_method_arguments,
 	void (*progress_start)(long i, long number, long size),
 	void (*progress)(long i, long number, long rcvd, long size, 
@@ -2047,11 +2023,7 @@ int pop3_retr(pop3_session_t *session,
 	void (*progress_abort)(long i, long number, long size),
 	char **errmsg, char **errstr)
 {
-    return pop3_delivery(session, 
-#if HAVE_SIGACTION
-	    abort, 
-#endif
-	    0, NULL, NULL,
+    return pop3_delivery(session, abort, 0, NULL, NULL,
 	    delivery_method, delivery_method_arguments,
 	    progress_start, progress, progress_end, progress_abort,
 	    errmsg, errstr);
@@ -2064,10 +2036,7 @@ int pop3_retr(pop3_session_t *session,
  * see pop3.h
  */
 
-int pop3_dele(pop3_session_t *session, 
-#if HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_dele(pop3_session_t *session, volatile sig_atomic_t *abort,
 	char **errmsg, char **errstr)
 {
     int e;
@@ -2099,13 +2068,11 @@ int pop3_dele(pop3_session_t *session,
 		    ? pipeline_min : 0);
 		recv_index++)
 	{
-#if HAVE_SIGACTION
     	    if (*abort)
     	    {
     		*errstr = xasprintf(_("operation aborted"));
     		return POP3_EABORT;
     	    }
-#endif
 	    if (session->msg_action[recv_index] != POP3_MSG_ACTION_DELETE)
 	    {
 		continue;
@@ -3171,5 +3138,5 @@ void pop3_close(pop3_session_t *session)
 	tls_close(&session->tls);
     }
 #endif /* HAVE_TLS */
-    net_close_socket(session->fd);
+    close(session->fd);
 }
