@@ -3,7 +3,7 @@
  * 
  * This file is part of mpop, a POP3 client.
  *
- * Copyright (C) 2003, 2004, 2005, 2006, 2007
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -24,10 +24,9 @@
 #define POP3_H
 
 #include <stdio.h>
-#ifdef HAVE_SIGACTION
-# include <signal.h>
-#endif
+#include <signal.h>
 
+#include "readbuf.h"
 #include "net.h"
 #ifdef HAVE_TLS
 # include "tls.h"
@@ -129,7 +128,6 @@ typedef struct
  * - new_size (= total_size)
  * The next function is pop3_uidl():
  * - msg_uid (UID of each mail)
- * - uids_sorted (indices so that the UIDs are sorted according to strcmp())
  * With this information, the caller can update the session fields of each mail:
  * Given a list of UIDs of previously retrieved mails, it can update:
  * - is_old (set true for retrieved mails)
@@ -179,7 +177,7 @@ typedef struct
     tls_t tls;			/* TLS descriptor */
 #endif /* HAVE_TLS */
     char buffer[POP3_BUFSIZE];	/* input/output buffer */
-    net_readbuf_t readbuf;	/* input buffering for 'fd', for net_gets() */
+    readbuf_t readbuf;		/* net input buffering */
     FILE *debug;		/* stream for debugging output, or NULL */
 
     /* POP3 session information */
@@ -192,7 +190,6 @@ typedef struct
     long new_size;		/* size of new messages */
     unsigned char *msg_action;	/* action for each mail */
     char **msg_uid;		/* UID of each mail */
-    long *uids_sorted;		/* indices so that the UIDs are sorted */
     long *msg_size;		/* size of each mail */
 } pop3_session_t;
 
@@ -280,9 +277,11 @@ int pop3_capa(pop3_session_t *session, char **errstr);
  * Used error codes: TLS_ELIBFAILED, TLS_EFILE
  */
 #ifdef HAVE_TLS
-int pop3_tls_init(pop3_session_t *session, const char *tls_key_file, 
-	const char *tls_ca_file, const char *tls_trust_file, 
-	int force_sslv3, char **errstr);
+int pop3_tls_init(pop3_session_t *session, 
+	const char *tls_key_file, const char *tls_cert_file,
+	const char *tls_trust_file, const char *tls_crl_file,
+	int force_sslv3, int min_dh_prime_bits, const char *priorities,
+	char **errstr);
 #endif /* HAVE_TLS */
 
 /*
@@ -388,11 +387,8 @@ int pop3_stat(pop3_session_t *session, char **errmsg, char **errstr);
  * The error POP3_EUNAVAIL is not critical: it means that the POP3 server does
  * not support the UIDL command.
  */
-int pop3_uidl(pop3_session_t *session,
-#ifdef HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
-	char **errmsg, char **errstr);
+int pop3_uidl(pop3_session_t *session, char **uidv, long uidv_len, int only_new,
+	volatile sig_atomic_t *abort, char **errmsg, char **errstr);
 
 /*
  * pop3_list()
@@ -404,10 +400,7 @@ int pop3_uidl(pop3_session_t *session,
  * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL
  */
-int pop3_list(pop3_session_t *session,
-#ifdef HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_list(pop3_session_t *session, volatile sig_atomic_t *abort,
 	char **errmsg, char **errstr);
 
 /*
@@ -430,10 +423,7 @@ int pop3_list(pop3_session_t *session,
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL, POP3_EDELIVERY,
  * POP3_EABORT.
  */
-int pop3_filter(pop3_session_t *session, 
-#ifdef HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_filter(pop3_session_t *session, volatile sig_atomic_t *abort,
 	const char *filtercmd,
 	void (*filter_output)(long i, long number, int new_action, void *data),
 	void *data, char **errmsg, char **errstr);
@@ -460,10 +450,7 @@ int pop3_filter(pop3_session_t *session,
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL, POP3_EDELIVERY,
  * POP3_EABORT
  */
-int pop3_retr(pop3_session_t *session,
-#ifdef HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_retr(pop3_session_t *session, volatile sig_atomic_t *abort,
 	int delivery_method, const char *delivery_method_arguments,
 	void (*progress_start)(long i, long number, long size),
 	void (*progress)(long i, long number, long rcvd, long size, 
@@ -483,10 +470,7 @@ int pop3_retr(pop3_session_t *session,
  * POP3_EABORTED. The POP3 session is not usable thereafter.
  * Used error codes: POP3_EIO, POP3_EPROTO, POP3_EINVAL
  */
-int pop3_dele(pop3_session_t *session, 
-#ifdef HAVE_SIGACTION
-	volatile sig_atomic_t *abort,
-#endif
+int pop3_dele(pop3_session_t *session, volatile sig_atomic_t *abort,
 	char **errmsg, char **errstr);
     
 /*
